import { Event } from "@codingame/monaco-vscode-api/vscode/vs/base/common/event";
import { URI } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { Disposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { IStorageService } from "@codingame/monaco-vscode-api/vscode/vs/platform/storage/common/storage.service";
import { IOutputChannel, OutputChannelUpdateMode, IOutputChannelDescriptor, IOutputViewFilters, ILogEntry } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/output/common/output";
import { IOutputService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/output/common/output.service";
import { ITextModelContentProvider } from "@codingame/monaco-vscode-model-service-override/vscode/vs/editor/common/services/resolverService";
import { ITextModelService } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/services/resolverService.service";
import { ITextModel } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/model";
import { LogLevel } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log";
import { ILoggerService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { ILogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { ILifecycleService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/lifecycle/common/lifecycle.service";
import { IOutputChannelModel } from "../common/outputChannelModel.js";
import { IViewsService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/views/common/viewsService.service";
import { ILanguageService } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/languages/language.service";
import { IContextKeyService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey.service";
import { IDefaultLogLevelsService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/logs/common/defaultLogLevels.service";
import { IFileDialogService } from "@codingame/monaco-vscode-api/vscode/vs/platform/dialogs/common/dialogs.service";
import { IFileService } from "@codingame/monaco-vscode-api/vscode/vs/platform/files/common/files.service";
import { IWorkbenchEnvironmentService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/environment/common/environmentService.service";
declare class OutputChannel extends Disposable implements IOutputChannel {
    readonly outputChannelDescriptor: IOutputChannelDescriptor;
    private readonly outputLocation;
    private readonly outputDirPromise;
    private readonly languageService;
    private readonly instantiationService;
    scrollLock: boolean;
    readonly model: IOutputChannelModel;
    readonly id: string;
    readonly label: string;
    readonly uri: URI;
    constructor(outputChannelDescriptor: IOutputChannelDescriptor, outputLocation: URI, outputDirPromise: Promise<void>, languageService: ILanguageService, instantiationService: IInstantiationService);
    private createOutputChannelModel;
    getLogEntries(): ReadonlyArray<ILogEntry>;
    append(output: string): void;
    update(mode: OutputChannelUpdateMode, till?: number): void;
    clear(): void;
    replace(value: string): void;
}
interface IOutputFilterOptions {
    filterHistory: string[];
    trace: boolean;
    debug: boolean;
    info: boolean;
    warning: boolean;
    error: boolean;
    sources: string;
}
declare class OutputViewFilters extends Disposable implements IOutputViewFilters {
    private readonly contextKeyService;
    private readonly _onDidChange;
    readonly onDidChange: Event<void>;
    constructor(options: IOutputFilterOptions, contextKeyService: IContextKeyService);
    filterHistory: string[];
    private _filterText;
    get text(): string;
    set text(filterText: string);
    private readonly _trace;
    get trace(): boolean;
    set trace(trace: boolean);
    private readonly _debug;
    get debug(): boolean;
    set debug(debug: boolean);
    private readonly _info;
    get info(): boolean;
    set info(info: boolean);
    private readonly _warning;
    get warning(): boolean;
    set warning(warning: boolean);
    private readonly _error;
    get error(): boolean;
    set error(error: boolean);
    private readonly _categories;
    get categories(): string;
    set categories(categories: string);
    toggleCategory(category: string): void;
    hasCategory(category: string): boolean;
}
export declare class OutputService extends Disposable implements IOutputService, ITextModelContentProvider {
    private readonly storageService;
    private readonly instantiationService;
    private readonly textModelService;
    private readonly logService;
    private readonly loggerService;
    private readonly lifecycleService;
    private readonly viewsService;
    private readonly defaultLogLevelsService;
    private readonly fileDialogService;
    private readonly fileService;
    readonly _serviceBrand: undefined;
    private readonly channels;
    private activeChannelIdInStorage;
    private activeChannel?;
    private readonly _onActiveOutputChannel;
    readonly onActiveOutputChannel: Event<string>;
    private readonly activeOutputChannelContext;
    private readonly activeFileOutputChannelContext;
    private readonly activeLogOutputChannelContext;
    private readonly activeOutputChannelLevelSettableContext;
    private readonly activeOutputChannelLevelContext;
    private readonly activeOutputChannelLevelIsDefaultContext;
    private readonly outputLocation;
    readonly filters: OutputViewFilters;
    constructor(storageService: IStorageService, instantiationService: IInstantiationService, textModelService: ITextModelService, logService: ILogService, loggerService: ILoggerService, lifecycleService: ILifecycleService, viewsService: IViewsService, contextKeyService: IContextKeyService, defaultLogLevelsService: IDefaultLogLevelsService, fileDialogService: IFileDialogService, fileService: IFileService, environmentService: IWorkbenchEnvironmentService);
    provideTextContent(resource: URI): Promise<ITextModel> | null;
    showChannel(id: string, preserveFocus?: boolean): Promise<void>;
    getChannel(id: string): OutputChannel | undefined;
    getChannelDescriptor(id: string): IOutputChannelDescriptor | undefined;
    getChannelDescriptors(): IOutputChannelDescriptor[];
    getActiveChannel(): IOutputChannel | undefined;
    canSetLogLevel(channel: IOutputChannelDescriptor): boolean;
    getLogLevel(channel: IOutputChannelDescriptor): LogLevel | undefined;
    setLogLevel(channel: IOutputChannelDescriptor, logLevel: LogLevel): void;
    registerCompoundLogChannel(descriptors: IOutputChannelDescriptor[]): string;
    saveOutputAs(...channels: IOutputChannelDescriptor[]): Promise<void>;
    private onDidRegisterChannel;
    private onDidUpdateChannelSources;
    private onDidRemoveChannel;
    private createChannel;
    private outputFolderCreationPromise;
    private instantiateChannel;
    private resetLogLevelFilters;
    private setLevelContext;
    private setLevelIsDefaultContext;
    private setActiveChannel;
}
export {};
