import { ICodeEditor } from "@codingame/monaco-vscode-api/vscode/vs/editor/browser/editorBrowser";
import { IEditorOptions as ICodeEditorOptions } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/config/editorOptions";
import { ITelemetryService } from "@codingame/monaco-vscode-api/vscode/vs/platform/telemetry/common/telemetry.service";
import { IStorageService } from "@codingame/monaco-vscode-api/vscode/vs/platform/storage/common/storage.service";
import { ITextResourceConfigurationService } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/services/textResourceConfiguration.service";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { IContextKeyService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextkey/common/contextkey.service";
import { IEditorOpenContext } from "@codingame/monaco-vscode-api/vscode/vs/workbench/common/editor";
import { AbstractTextResourceEditor } from "@codingame/monaco-vscode-523730aa-81e6-55d7-9916-87ad537fe087-common/vscode/vs/workbench/browser/parts/editor/textResourceEditor";
import { IOutputChannel } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/output/common/output";
import { IOutputService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/output/common/output.service";
import { IThemeService } from "@codingame/monaco-vscode-api/vscode/vs/platform/theme/common/themeService.service";
import { IConfigurationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/configuration/common/configuration.service";
import { IEditorGroupsService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorGroupsService.service";
import { CancellationToken } from "@codingame/monaco-vscode-api/vscode/vs/base/common/cancellation";
import { IEditorService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/editor/common/editorService.service";
import { IViewPaneOptions, FilterViewPane } from "@codingame/monaco-vscode-65619f8f-0eab-5d8b-855a-43b6353fe527-common/vscode/vs/workbench/browser/parts/views/viewPane";
import { IKeybindingService } from "@codingame/monaco-vscode-api/vscode/vs/platform/keybinding/common/keybinding.service";
import { IContextMenuService } from "@codingame/monaco-vscode-api/vscode/vs/platform/contextview/browser/contextView.service";
import { IViewDescriptorService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/common/views.service";
import { TextResourceEditorInput } from "@codingame/monaco-vscode-86d65fc6-30f9-5dca-9501-e249de688591-common/vscode/vs/workbench/common/editor/textResourceEditorInput";
import { IOpenerService } from "@codingame/monaco-vscode-api/vscode/vs/platform/opener/common/opener.service";
import { ITextEditorOptions } from "@codingame/monaco-vscode-api/vscode/vs/platform/editor/common/editor";
import { IFileService } from "@codingame/monaco-vscode-api/vscode/vs/platform/files/common/files.service";
import { IEditorConfiguration } from "@codingame/monaco-vscode-12c5f9eb-72d3-57ca-babd-5bef7aa9de3b-common/vscode/vs/workbench/browser/parts/editor/textEditor";
import { IHoverService } from "@codingame/monaco-vscode-api/vscode/vs/platform/hover/browser/hover.service";
import { Disposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { ICodeEditorWidgetOptions } from "@codingame/monaco-vscode-api/vscode/vs/editor/browser/widget/codeEditor/codeEditorWidget";
import { IEditorContribution } from "@codingame/monaco-vscode-api/vscode/vs/editor/common/editorCommon";
export declare class OutputViewPane extends FilterViewPane {
    private readonly outputService;
    private readonly editor;
    private channelId;
    private editorPromise;
    private readonly scrollLockContextKey;
    get scrollLock(): boolean;
    set scrollLock(scrollLock: boolean);
    private readonly memento;
    private readonly panelState;
    constructor(options: IViewPaneOptions, keybindingService: IKeybindingService, contextMenuService: IContextMenuService, configurationService: IConfigurationService, contextKeyService: IContextKeyService, viewDescriptorService: IViewDescriptorService, instantiationService: IInstantiationService, openerService: IOpenerService, themeService: IThemeService, hoverService: IHoverService, outputService: IOutputService, storageService: IStorageService);
    showChannel(channel: IOutputChannel, preserveFocus: boolean): void;
    focus(): void;
    clearFilterText(): void;
    protected renderBody(container: HTMLElement): void;
    protected layoutBodyContent(height: number, width: number): void;
    private onDidChangeVisibility;
    private setInput;
    private checkMoreFilters;
    private clearInput;
    private createInput;
    saveState(): void;
}
export declare class OutputEditor extends AbstractTextResourceEditor {
    private readonly configurationService;
    private readonly resourceContext;
    constructor(telemetryService: ITelemetryService, instantiationService: IInstantiationService, storageService: IStorageService, configurationService: IConfigurationService, textResourceConfigurationService: ITextResourceConfigurationService, themeService: IThemeService, editorGroupService: IEditorGroupsService, editorService: IEditorService, fileService: IFileService);
    getId(): string;
    getTitle(): string;
    protected getConfigurationOverrides(configuration: IEditorConfiguration): ICodeEditorOptions;
    protected getAriaLabel(): string;
    protected computeAriaLabel(): string;
    setInput(input: TextResourceEditorInput, options: ITextEditorOptions | undefined, context: IEditorOpenContext, token: CancellationToken): Promise<void>;
    clearInput(): void;
    protected createEditor(parent: HTMLElement): void;
    private _getContributions;
    protected getCodeEditorWidgetOptions(): ICodeEditorWidgetOptions;
}
export declare class FilterController extends Disposable implements IEditorContribution {
    private readonly editor;
    private readonly outputService;
    static readonly ID = "output.editor.contrib.filterController";
    private readonly modelDisposables;
    private hiddenAreas;
    private readonly categories;
    private readonly decorationsCollection;
    constructor(editor: ICodeEditor, outputService: IOutputService);
    private onDidChangeModel;
    private filter;
    private filterIncremental;
    private compute;
    private shouldShowLogLevel;
    private shouldShowCategory;
}
